# This file is part of Tarmac.
#
# Copyright 2011 Canonical, Ltd.
#
# Tarmac is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# Tarmac is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Tarmac.  If not, see <http://www.gnu.org/licenses/>.
"""Add a mode to ignore all branches merges apart from selected one."""

from tarmac.exceptions import TarmacMergeIgnore
from tarmac.hooks import tarmac_hooks
from tarmac.plugins import TarmacPlugin

import os

UNIGNORE_KEYWORD = "UNBLOCK"

class IgnoreMerge(TarmacMergeIgnore):
    """Error for when a jenkins is not reachable."""

class IgnoreMergeMode(TarmacPlugin):
    """Tarmac plug-in to ignore merges on branches apart from selected ones

    This plug-in block merges if (a) specific file(s) exists, then only branches
    containing "UNBLOCK" as capital letter in their merge description will still
    continue merging.
    Set ignoremerge_on to your configuration for the current target to wait
    for this file to disappear to process merging every branches.
    Multiples files can be separated by ;
    """
    
    def run(self, command, target, source, proposal):
        """Wait on the existence of other files"""
        
        try:
            files = getattr(target.config, "ignoremerge_on").split(";")
        except AttributeError:
            return
            
        if self._in_ignore_mode(files):
            if self._branch_should_be_ignored(proposal.description):
                message = (u'Ignoring the branch %(source)s because on of %(block)s '
                           u'files exists on disc and the description doesn\'t '
                           u'contain %(magicword)s magical word' % {
                            'source': proposal.source_branch.display_name,
                            'block': files,
                            'magicword': UNIGNORE_KEYWORD})
                raise IgnoreMerge(message)
        
    def _in_ignore_mode(self, files):
        '''detect if we are in ignore mode'''
        for blocking_file in files:
            if os.path.exists(blocking_file):
                return True
        return False
        
    def _branch_should_be_ignored(self, description):
        """Tell that the branch should not be ignored if contains UNBLOCK in the merge description"""
        return not (UNIGNORE_KEYWORD in description)
        

tarmac_hooks['tarmac_pre_commit'].hook(
    IgnoreMergeMode(), 'Ignore merge mode control')
