# Copyright 2011 Canonical, Ltd.
#
# This file is part of Tarmac.
#
# Tarmac is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# Tarmac is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Tarmac.  If not, see <http://www.gnu.org/licenses/>.
"""Tests ignore merge mode plug-in."""

from tarmac.plugins.ignoremergemode import IgnoreMergeMode, IgnoreMerge, UNIGNORE_KEYWORD
from tarmac.tests import TarmacTestCase
from tarmac.tests.mock import Thing

import os
import tempfile

class IgnoreMergeModeTests(TarmacTestCase):
    """Test the ignore merge mode plugin."""
    
    
    def setUp(self):
        super(IgnoreMergeModeTests, self).setUp()
        self.ignoremergefile = tempfile.mkstemp()[1]
        self.plugin = IgnoreMergeMode()
        self.proposal = Thing(
            source_branch = Thing(display_name='lp:foo/bar'),
            description = 'foo %s bar' % UNIGNORE_KEYWORD)
        self.target = Thing(config=Thing(ignoremerge_on=self.ignoremergefile))
        
    def tearDown(self):
        super(IgnoreMergeModeTests, self).tearDown()
        try:
            os.remove(self.ignoremergefile)
        except OSError:
            pass
            
    def test_no_block(self):
        """Test that with no ignore mode file, we don't reject the branch"""
        self.target = Thing(config=Thing())
        # should not raise anything
        self.plugin.run(command=None, target=self.target, source=None,
                        proposal=self.proposal)
        
    def test_ignore_branch(self):
        """Ensure an ordinary branch raise an exception and so, ignored"""
        self.proposal.description = "foo"
        self.assertRaises(IgnoreMerge, self.plugin.run, command=None,
                          target=self.target, source=None,
                          proposal=self.proposal)
                          
    def test_unignored_branch(self):
        """Ensure the special branch that can pass the merge ignore check"""
        self.plugin.run(command=None, target=self.target, source=None,
                        proposal=self.proposal)
                        
    def test_in_ignore_mode(self):
        """Test that we are in ignored mode with one file"""
        self.assertTrue(self.plugin._in_ignore_mode([self.ignoremergefile]))
        
    def test_ignore_mode_with_two_files(self):
        """Test that we are in ignore mode with two files, one not existing"""
        self.assertTrue(self.plugin._in_ignore_mode(["abcd", self.ignoremergefile]))
        
    def test_not_ignore_mode(self):
        """Test that we are not in ignore mode with two unexisting files, one empty"""
        self.assertFalse(self.plugin._in_ignore_mode(["abcd", ""]))
        
    def test_ignored_branch(self):
        """Test that a ignored branch should be ignored"""
        self.assertTrue(self.plugin._branch_should_be_ignored("foo"))
        
    def test_unignored_branch(self):
        """Test that a branch with the right keyword should not be ignored"""
        self.assertFalse(self.plugin._branch_should_be_ignored("foo %s" % UNIGNORE_KEYWORD))
