# Copyright 2011 Canonical, Ltd.
#
# This file is part of Tarmac.
#
# Tarmac is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# Tarmac is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Tarmac.  If not, see <http://www.gnu.org/licenses/>.
"""Tests for the Jenkins Integration plug-in."""

from tarmac.plugins.jenkins import JenkinsCheck, JenkinsTimeout, JenkinsReject
from tarmac.tests import TarmacTestCase
from tarmac.tests.mock import Thing

import os
import shutil
import tempfile

class JenkinsCheckTests(TarmacTestCase):
    """Test the jenkins integration."""

    def setUp(self):
        super(JenkinsCheckTests, self).setUp()
        self.jenkins_dir = tempfile.mkdtemp()
        self.jenkins_token = "footoken"
        config=Thing(jenkins_job_url="file://%s" % self.jenkins_dir,
                     packaging_branch='lp:~foo/test/ubuntu',
                     jenkins_token=self.jenkins_token,
                     jenkins_public_url="fooo")
        self.target = Thing(config=config, tree=Thing(abspath=os.path.abspath),
                            bzr_branch=Thing(revno=lambda: 10))
        self.proposal = Thing(
            source_branch = Thing(display_name='lp:foo/bar'))
        self.command = Thing()
        self.plugin = JenkinsCheck()
        
    def tearDown(self):
        super(JenkinsCheckTests, self).tearDown()
        shutil.rmtree(self.jenkins_dir)
        
    def generate_jenkins_mock_server(self, run_status=None, building=None, last_job_status=None):
        if run_status:
            run_test_file = ('%(url)s/buildWithParameters?token=%(token)s&dir=%(dir)s&branch=%(branch)s&trunkrev=%(revno)s'
                   % {'url': self.jenkins_dir, 'token': self.jenkins_token, 'dir': self.target.tree.abspath(''),
                      'revno': self.target.bzr_branch.revno(), 'branch': self.target.config.packaging_branch})
            os.makedirs(os.path.dirname(run_test_file))
            with open(run_test_file, 'w') as f:
                f.write("")
        if building or last_job_status:
            os.makedirs(self.jenkins_dir + '/lastBuild/api/')
            with open(self.jenkins_dir + '/lastBuild/api/json', 'w') as f:
                f.write('{"actions":[{"parameters":[{"name":"RELEASE","value":"oneiric"},{"name":"VARIANT","value":"alternate"},{"name":"ARCH","value":"i386"},{"name":"PRESEED","value":"oem"},{"name":"FLAVOR","value":"ubuntu"}]},{"causes":[]},{}],"artifacts":[],"building":%(building)s,"description":null,"duration":2701176,"fullDisplayName":"oneiric-alternate-i386_oem #65","id":"2011-11-07_07-57-49","keepLog":false,"number":65,"result":"%(last_job_status)s","timestamp":1320652669000,"url":"http://jenkins.qa.ubuntu.com/job/oneiric-alternate-i386_oem/65/","builtOn":"aldebaran","changeSet":{"items":[],"kind":null},"culprits":[]}' % {'building': building, 'last_job_status': last_job_status})
        
    def test_fail_no_server(self):
        """Test that the merge actually fail when there is no server."""
        # no file
        self.assertRaises(JenkinsTimeout, self.plugin.run, command=self.command,
                          target=self.target, source=None,
                          proposal=self.proposal)
                            
    def test_fail_running_doesnt_answer(self):
        """Test that the merge actually fail when is_running() doesn't answer
           and don't stall."""
        # no "running" file
        self.generate_jenkins_mock_server(run_status='OK')
        self.assertRaises(JenkinsTimeout, self.plugin.run, command=self.command,
                          target=self.target, source=None,
                          proposal=self.proposal)
                          
    def test_pass(self):
        """Test that nothing is raised if all finished as excpected."""
        self.generate_jenkins_mock_server(run_status='OK', building = 'false',
                                          last_job_status='SUCCESS')
        self.plugin.run(command=self.command,
                        target=self.target, source=None,
                        proposal=self.proposal)
                        
    def test_jenkins_not_passed(self):
        """Test that the merge actually fail if jenkins last job failed"""
        self.generate_jenkins_mock_server(run_status='OK', building = 'false',
                                          last_job_status='FAILURE')
        self.assertRaises(JenkinsReject, self.plugin.run, command=self.command,
                          target=self.target, source=None,
                          proposal=self.proposal)
                          
    def test_no_jenkins_parameter(self):
        """Test that nothing happens if there is no jenkins parameter"""
        self.target = Thing(config=Thing())
        self.plugin.run(command=self.command, target=self.target, source=None,
                        proposal=self.proposal)

    def test_jenkins_fail_if_only_jenkins_url(self):
        """Test that jenkins fails if there is only the jenkins url but no other mandatory parameters"""
        self.target = Thing(config=Thing(jenkins_job_url="file://%s" % self.jenkins_dir))
        self.assertRaises(JenkinsTimeout, self.plugin.run, command=self.command,
                          target=self.target, source=None,
                          proposal=self.proposal)

    def test_no_jenkins_public_url(self):
        """Test that if no public url is given to jenkins, the jenkins job is used"""
        config = Thing(jenkins_job_url="file://%s" % self.jenkins_dir,
                       packaging_branch='lp:~foo/test/ubuntu',
                      jenkins_token=self.jenkins_token)
        self.target = Thing(config=config, tree=Thing(abspath=os.path.abspath),
                            bzr_branch=Thing(revno=lambda: 10))
        self.generate_jenkins_mock_server(run_status='OK', building = 'false',
                                          last_job_status='SUCCESS')
        self.plugin.run(command=self.command,
                        target=self.target, source=None,
                        proposal=self.proposal)
        self.assertEqual(self.plugin.jenkins_job.url, self.plugin.jenkins_job.public_url)

