# Copyright 2011 Canonical, Ltd.
#
# This file is part of Tarmac.
#
# Tarmac is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# Tarmac is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Tarmac.  If not, see <http://www.gnu.org/licenses/>.
"""Tests handing build-dep when multiple tarmacs are running in parallel plug-in."""

from tarmac.plugins.mergedeponfile import MergeDepOnFile, TIME_BETWEEN_TWO_CHECKS
from tarmac.tests import TarmacTestCase
from tarmac.tests.mock import Thing

import os
import tempfile
import time

class MergedepOnFileTests(TarmacTestCase):
    """Test the merge dependency on plugin."""
    
    
    def setUp(self):
        super(MergedepOnFileTests, self).setUp()
        self.depfile = tempfile.mkstemp()[1]
        self.plugin = MergeDepOnFile()
        
    def tearDown(self):
        super(MergedepOnFileTests, self).tearDown()
        try:
            os.remove(self.depfile)
        except OSError:
            pass
                
    def test_no_dep(self):
        """Test that with no build-dep, we don't wait at all"""
        start_time = time.time()
        self.plugin.run(command=None, target=None)
        end_time = time.time()
        self.assertTrue(end_time - start_time < TIME_BETWEEN_TWO_CHECKS)
        
    def test_no_dep_but_option(self):
        """Test that with no build-dep, but the option checked, we wait just once"""
        target = Thing(config = Thing(mergedep_on="abcdefgh"))
        start_time = time.time()
        self.plugin.run(command=None, target=target)
        end_time = time.time()
        self.assertTrue(end_time - start_time > TIME_BETWEEN_TWO_CHECKS)
        self.assertTrue(end_time - start_time < 2*TIME_BETWEEN_TWO_CHECKS)

    def test_no_dep_but_option_one_empty(self):
        """Test that with no build-dep, but the option checked and an empty file, we wait just once"""
        target = Thing(config = Thing(mergedep_on="abcdefgh;"))
        start_time = time.time()
        self.plugin.run(command=None, target=target)
        end_time = time.time()
        self.assertTrue(end_time - start_time > TIME_BETWEEN_TWO_CHECKS)
        self.assertTrue(end_time - start_time < 2*TIME_BETWEEN_TWO_CHECKS)
        
    def test_one_dep_only(self):
        """Test that with one build-dep, we wait until test_timeout_run"""
        target = Thing(config = Thing(mergedep_on=self.depfile,
                                      test_timeout_run=2))
        start_time = time.time()
        self.plugin.run(command=None, target=target)
        end_time = time.time()
        self.assertTrue(end_time - start_time > 2*TIME_BETWEEN_TWO_CHECKS)
        self.assertTrue(end_time - start_time < 3*TIME_BETWEEN_TWO_CHECKS)

    def test_two_deps_but_one_file(self):
        """Test that with one build-dep on two, we wait until test_timeout_run"""
        target = Thing(config = Thing(mergedep_on="abcdefgh;%s" % self.depfile,
                                      test_timeout_run=2))
        start_time = time.time()
        self.plugin.run(command=None, target=target)
        end_time = time.time()
        self.assertTrue(end_time - start_time > 2*TIME_BETWEEN_TWO_CHECKS)
        self.assertTrue(end_time - start_time < 3*TIME_BETWEEN_TWO_CHECKS)
        
